/** 
**************************************************************************
* @file   modbus_data.h
* @brief    Заголовочный файл с описанием даты MODBUS.
* @details  Данный файл необходимо подключается в rs_message.h. После rs_message.h 
* подключается к основному проекту.
* 
* @defgroup MODBUS_DATA
* @ingroup  MODBUS
* @brief    Modbus data description
*
*************************************************************************/

#ifndef _MODBUS_DATA_H_
#define _MODBUS_DATA_H_
  
#include "stdint.h"
//--------------DEFINES FOR REGISTERS---------------
// DEFINES FOR ARRAYS
/** 
  * @addtogroup MODBUS_DATA_RERISTERS_DEFINES
  * @ingroup    MODBUS_DATA
  * @brief      Defines for registers
  Структура дефайна адресов
  @verbatim    
  Для массивов регистров:
    R_<NAME_ARRAY>_ADDR   -  модбас адресс первого регистра в массиве
    R_<NAME_ARRAY>_QNT    -  количество регистров в массиве
  
  @endverbatim
  * @{
  */
  
  
/** 
  * @brief Регистры хранения
  */
typedef struct //MB_DataInRegsTypeDef
{
  uint16_t ForwardVoltage;
  uint16_t ReversePeakVoltage;
}MB_DataInRegsTypeDef;


/** 
  * @brief Входные регистры 
  */
typedef struct //MB_DataInRegsTypeDef
{
  uint16_t TimeForForward;
  uint16_t TimeBeforeTest;
  uint16_t TimeDeadtime;
  uint16_t TimeBeforePeak;
  uint16_t TimeBeforeDisconnect;
  
  uint16_t Adc_PulseWidth;
  uint16_t Adc_PulseSign;
  uint16_t Adc_CalibrValue;
  uint16_t Adc_ZeroValue;
  uint16_t Adc_U_Calibr;
}MB_DataHoldRegsTypeDef;


// DEFINES FOR INPUT REGISTERS ARRAYS
#define   R_MEASURED_ADDR                 0
#define   R_MEASURED_QNT                  2

// DEFINES FOR HOLDING REGISTERS ARRAYS
#define   R_SETTINGS_ADDR                   0
#define   R_SETTINGS_QNT                    10


// DEFINES FOR REGISTERS LOCAL ADDRESSES
//#define R_SET_ERROR(_te_num_)          0

 
/** MODBUS_DATA_RERISTERS_DEFINES
  * @} 
  */

//----------------DEFINES FOR COILS-----------------
/** 
  * @addtogroup MODBUS_DATA_COILS_DEFINES
  * @ingroup    MODBUS_DATA
  * @brief      Defines for coils
  @verbatim    
  Структура дефайна
  Для массивов коилов:
    C_<NAME_ARRAY>_ADDR   - модбас адресс первого коила в массиве
    C_<NAME_ARRAY>_QNT    - количество коилов в массиве (минимум 16)

  @endverbatim
  * @{
  */
  
  
  
/** 
  * @brief     Коилы
  */
typedef struct //MB_DataCoilsTypeDef
{
  /* reg 1 - control */
  unsigned ForwardTest:1;
  unsigned ReverseTest:1;
  unsigned StartTest:1;
  unsigned PhaseForwardHiTest:1;
  unsigned PhaseForwardLoTest:1;
  unsigned PhaseReverseHiTest:1;
  unsigned PhaseReverseLoTest:1;
  
  unsigned reserved:9;
  
  /* reg 2 - settings */
  unsigned msTimeBeforeTest_enable:1;
  unsigned msTimeDeadtime_enable:1;
  unsigned msTimeBeforePeak_enable:1;
  unsigned msTimeBeforeDisconnect_enable:1;  
  unsigned reserved2:11;
}MB_DataCoilsTypeDef;

// DEFINES FOR COIL ARRAYS
#define C_CONTROL_ADDR                0
#define C_CONTROL_QNT                 7

#define C_SETTINGS_ADDR               16
#define C_SETTINGS_QNT                3

/** MODBUS_DATA_COILS_DEFINES
  * @} 
  */
  
  
//-----------MODBUS DEVICE DATA SETTING-------------
// MODBUS DATA STRUCTTURE
/** 
  * @brief Структура со всеми регистрами и коилами модбас 
  * @ingroup  MODBUS_DATA
  */
typedef struct // tester modbus data
{
  MB_DataInRegsTypeDef      InRegs;     ///< Modbus input registers @ref MB_DataInRegsTypeDef

  MB_DataCoilsTypeDef       Coils;      ///< Modbus coils @ref MB_DataCoilsTypeDef
  
  MB_DataHoldRegsTypeDef    HoldRegs;   ///< Modbus holding registers @ref MB_DataHoldRegsTypeDef
}MB_DataStructureTypeDef;
extern MB_DataStructureTypeDef MB_DATA;



#endif //_MODBUS_DATA_H_

/////////////////////////////////////////////////////////////
///////////////////////TEMP/OUTDATE/OTHER////////////////////

//typedef enum //MB_TESTERCommandsTypeDef
//{
//  StandartMode       = 0x00, ///< Стандартная работа
//  Opros_TE_Disable   = 0x01, ///< Запрет опроса ТЭ (активен только обмен с ЛСУ ЭС, ТЭ не контролируются)
//}MB_TESTERCommandsTypeDef;

///**
//  * @brief     Состояние МЗКТЭ
//  */
//typedef enum //MB_TESTERErrStatusTypeDef
//{
//  TESTER_OK           = 0x0,   ///< МЗКТЭ функционирует нормально. Идет опрос ТЭ.
//  NonCritical_Err    = 0x1,  ///< Неисправность МЗКТЭ, при которой МЗКТЭ может выполнять свои основные функции (некоторые программные ошибки из @ref MB_TESTERTrackerTypeDef).
//  Critical_Err       = 0x2,  ///< Неисправность МЗКТЭ, при которой выполнение основных функций не представляется возможным (ошибки 1-3 и  некоторые программные ошибки из @ref MB_TESTERTrackerTypeDef)
//  
//}MB_TESTERErrStatusTypeDef;
//typedef enum
//{
//  TE_No_Err   = 0x0,   ///< Напряжения на всех ТЭ выше аварийных порогов, задаваемых уставками «Авария»
//  TE_Err      = 0x1,  ///< Напряжение на одном или нескольких ТЭ достигло или ниже аварийного порога, задаваемого уставкой «Авария»
//}MB_TEErrActiveTypeDef;
//typedef enum
//{
//  TE_No_Warn   = 0x0,   ///< Напряжения на всех ТЭ выше предупредительных порогов, задаваемых уставкой «Предупреждение»
//  TE_Warn      = 0x1,  ///< Напряжение на одном или нескольких ТЭ достигло или ниже предупредительного порога, задаваемого уставкой «Предупреждение»
//}MB_TEWarnActiveTypeDef;
//typedef enum
//{
//  OprosTE_Enable   = 0x0,   ///< Опрос ТЭ разрешен
//  OprosTE_Disable      = 0x1,  ///< Опрос ТЭ запрещен  (см. регистр хранения 170)
//}MB_OprosTETypeDef;
