/**
**************************************************************************
* @file tester_config.h
* @brief Конфигурации для тестера диодов
**************************************************************************
* @defgroup    TESTER_CONFIGS    Configs for project
* @brief       Конфигурации для тестера диодов
* @details     
@{
*************************************************************************/
#ifndef _TESTER_CONFIG_H_
#define _TESTER_CONFIG_H_


/** 
	* @addtogroup TESTER_SW_TIMINGS_CONFIG   Configs for switching timings
	* @ingroup 		TESTER_CONFIGS
  * @brief      Конфигурации таймингов для ключей питания
	@{
	*/

/**
  * @brief    Задержка (миллисекундная) для положительного напряжения ПО УМОЛЧАНИЮ
  * @details  Пока только миллисекунды, т.к. меньше я пока не реализовал, да и как понимаю не требуется
  */
#define DEF_MS_TIME_FOR_POSITIVE              2000

/**
  * @brief    Задержка для дедтайма ПО УМОЛЧАНИЮ
  * @details  Задержка миллисекундная или микросекундная @ref DEF_DEADTIME_MS_DELAY
  */
#define DEF_DEADTIME                          500
#define DEF_DEADTIME_MS_DELAY                 0         ///< включение миллисекундной задержки для @ref DEF_DEADTIME по умолчанию

/**
  * @brief    Задержка перед началом тестирования ПО УМОЛЧАНИЮ
  * @details  Задержка миллисекундная или микросекундная @ref TIME_BEFORE_TEST_MS_DELAY
  */
#define DEF_TIME_BEFORE_TEST                  500
#define DEF_TIME_BEFORE_TEST_MS_DELAY         1         ///< включение миллисекундной задержки для @ref TIME_BEFORE_TEST по умолчанию

/**
  * @brief    Задержка между включением АЦП и предполагаемым скачком напряжения ПО УМОЛЧАНИЮ
  * @details  Задержка нужна, чтобы точно поймать его в буфере АЦП. 
  *
  *           Задержка миллисекундная или микросекундная @ref TIME_BEFORE_TEST_MS_DELAY  
  * @note     Но если миллисекундная, то скорее всего скачок не попадет в буфер АЦП.
  *           Поэтому желательно только микросекундная (@ref TIME_BEFORE_PEAK_MS_DELAY = 0)
  */
#define DEF_TIME_BEFORE_PEAK                  5
#define DEF_TIME_BEFORE_PEAK_MS_DELAY         0         ///< включение миллисекундной задержки для @ref TIME_BEFORE_PEAK

/**
  * @brief    Задержка перед окончанием тестирования (отключение питания) ПО УМОЛЧАНИЮ
  * @details  Задержка миллисекундная или микросекундная @ref TIME_BEFORE_DISCONNECT_MS_DELAY
  */
#define DEF_TIME_BEFORE_DISCONNECT            2000
#define DEF_TIME_BEFORE_DISCONNECT_MS_DELAY   1         ///< включение миллисекундной задержки для @ref TIME_BEFORE_DISCONNECT

/** TESTER_SW_TIMINGS_CONFIG
  * @}
  */


/** 
	* @addtogroup TESTER_INTERFACE_CONFIG   Configs for interface of tester
	* @ingroup 		TESTER_CONFIGS
  * @brief      Конфигурации для интерфейса тестера
	@{
	*/

/* Дефайны для светодиодов */
/* Состояния включенного и выключенного светодиода */
#define LED_ON      0     ///< Состояние включенного светодиода
#define LED_OFF     1     ///< Состояние выключенного светодиода

#define LED1_Port                       GPIOC         ///< Порт светодиода (при перенастройке надо также перенастроить через cube)
#define LED1_Pin                        GPIO_PIN_13   ///< Пин светодиода (при перенастройке надо также перенастроить через cube)

/* Периоды моргания светодиода */
#define LED_FORWARD_DIODE_PERIOD        250           ///< Моргание для индикации подключенного положительного напряжения
#define LED_REVERSE_DIODE_PERIOD        25            ///< Моргание для индикации подключенного отрицательного напряжения
#define LED_BLINK_AS_ON                 5             ///< Моргание незаметное для глаза (светодиод просто включен)


/* Дефайны для кнопки */
/* Состояния нажатой и отжатой кнопки */
#define SW_ON       1     ///< Состояние нажатой кнопки
#define SW_OFF      0     ///< Состояние отжатой кнопки

/* Пин кнопки при перенастройке надо также перенастроить через cube) */
#define SWITCH_START_Port               GPIOC         ///< Порт кнокпи старта (при перенастройке надо также перенастроить через cube)
#define SWITCH_START_Pin                GPIO_PIN_14   ///< Пин кнокпи старта (при перенастройке надо также перенастроить через cube)

/* Задержка для компенсации дребезга (в миллисекундах) */
#define SWITCH_ANTI_DREBEZG_DELAY       50            ///< Время которое выжидается для окончания дребезга

/** TESTER_INTERFACE_CONFIG
  * @}
  */

/** 
	* @addtogroup TESTER_PHASE_SW_CONFIG   Configs for switches for phases
	* @ingroup 		TESTER_CONFIGS
  * @brief      Конфигурации для ключей фаз
	@{
	*/
//#define USE_HAL_GPIO_FUNCTIONS  ///< По этому дефайну для переключения пинов будут использоваться HAL_GPIO функции
//#define RECONNECT_WITHOUT_DEADTIME ///< По этому дефайну дедтайм отключается. Если отключить и USE_HAL_GPIO_FUNCTIONS, то переключатся фазы будут почти синхронно (160 нс)
  
/* Состояния ключей для подключения и откючения питания */
#define PHASE_CONNECT       1   ///< Ключ замкнут подключено в данном состоянии пина
#define PHASE_DISCONNECT    0   ///< Ключ разомкнут отключено в данном состоянии пина

/* Ключи для прямого включения диода */
/* (при перенастройке надо также перенастроить через cube) */
#define SWITCH_PHASE_FORWARD_Port       GPIOB         ///< Порт пина первого ключа для прямого включения диода
#define SWITCH_PHASE_FORWARD_HI_Pin     GPIO_PIN_0    ///< Пин первого ключа для прямого включения диода
#define SWITCH_PHASE_FORWARD_LO_Pin     GPIO_PIN_1    ///< Пин второго ключа для прямого включения диода


/* Ключи для обратного включения диода */
/* (при перенастройке надо также перенастроить через cube) */
#define SWITCH_PHASE_REVERSE_Port       GPIOB         ///< Порт пина первого ключа для обратного включения диода
#define SWITCH_PHASE_REVERSE_HI_Pin     GPIO_PIN_10   ///< Пин первого ключа для обратного включения диода
#define SWITCH_PHASE_REVERSE_LO_Pin     GPIO_PIN_11   ///< Пин первого ключа для обратного включения диода

/** TESTER_PHASE_SW_CONFIG
  * @}
  */

/** 
	* @addtogroup TESTER_ADC_CONFIG   Configs for ADC
	* @ingroup 		TESTER_CONFIGS
  * @brief      Конфигурации для АЦП
	@{
	*/
  
#define ADC_BUFF_SIZE                           (200)               ///< Размер буфера АЦП
#define ADC_DMA_BUFF_SIZE                       (ADC_BUFF_SIZE)     ///< Размер буфера ДМА (1 канал)

#define ADC_U_CALIBR                            ((float)3.0)        ///< Калибровочное напряжение
#define ADC_VALUE_CALIBR                        (0xFFF)             ///< Значение АЦП при калибровочном напряжении @ref ADC_U_CALIBR
#define ADC_VALUE_ZERO                          (0)                 ///< Значение АЦП при нулевом напряжении
  
#define ADC_READ_TIMEOUT_MS                     20                  ///< Таймаут на ожидание разрядки конденсатора

#define TESTER_ADC_PULES_EXPETCED_WIDTH   4   ///< Предполагаемая длительность пика в отчетах ацп

/** TESTER_ADC_CONFIG
  * @}
  */


/** TESTER_CONFIGS
  * @}
  */

#endif //_TESTER_CONFIG_H_

