#include "tester_func.h"

TESTER_TestHandleTypeDef hTestDiode;

/**s
  * @brief    Инициализация хендла тестера диодов
  */
void TESTER_HandleInit(TESTER_TestHandleTypeDef *htest, TESTER_LEDsTypeDef *leds)
{
  /* Настройка структуры АЦП */
  htest->adc = &tester_adc;  
  htest->leds = leds;
  TESTER_ADC_StructInit(htest->adc);
    
  htest->continuous_buff_size = 20;
}


/**
  * @brief    Тест диодов: подключение прямого напряжения
  */
void TESTER_TestDiode_Forward(TESTER_TestHandleTypeDef *htest)
{
  /* Отключение питания от диода */
  TESTER_Disconnect_AllPhases(&htest->SwPhaseReverse, &htest->SwPhaseForward);  
  /* Очистка буферов каналов */
  ClearStruct(htest->adc->chAdc.ADC_Buff);
  
  /* Задержка, перед началом работы */
  TESTER_LED_TimeoutForStartTest(&htest->leds->LED1);
  TESTER_Delay(&htest->SwTimings.ticks_before_test, &hmcstim);
  TESTER_LED_TestingDiodeForward(&htest->leds->LED1);
  
  /* Включение континиус АЦП */
  HAL_ADC_Start(htest->adc->hadc);  
  
  /* Прямое включение диода */
  TESTER_Connect_Phase(&htest->SwPhaseForward);
  
  /* Прямое включение на определенное время */
  uint32_t tickstart = HAL_GetTick();
  HAL_StatusTypeDef res;
  while(1)
  {
    /* Считывание АЦП пока таймаут истечет */
    res = ADC_ReadContinuous(htest->adc, htest->continuous_buff_size, tickstart, htest->SwTimings.msticks_for_forward);
    
    if(res != HAL_OK)
      break;
  }
  
  /* Отключение питания от диода */
  TESTER_Disconnect_AllPhases(&htest->SwPhaseReverse, &htest->SwPhaseForward);
  
  /* Выключение континиус АЦП */
  HAL_ADC_Stop(htest->adc->hadc);
  
  TESTER_LED_WaitForAction(&htest->leds->LED1);  
  
  htest->DiodeForwardVolt = htest->adc->chAdc.U_Current;
}

/**
  * @brief    Тест диодов: подключение обратного напряжения
  */
void TESTER_TestDiode_Reverse(TESTER_TestHandleTypeDef *htest)
{
  /* Отключение питания от диода */
  TESTER_Disconnect_AllPhases(&htest->SwPhaseReverse, &htest->SwPhaseForward);
  /* Очистка буферов каналов */
  ClearStruct(htest->adc->chAdc.ADC_Buff);
  
  /* Задержка, перед началом работы */
  TESTER_LED_TimeoutForStartTest(&htest->leds->LED1);
  TESTER_Delay(&htest->SwTimings.ticks_before_test, &hmcstim);
  TESTER_LED_TestingDiodeReverse(&htest->leds->LED1);
  
  /* Включение АЦП */
  ADC_DMA_StartRead(htest->adc);
  
  /* Задержка, перед предполагаемым скачком */
  TESTER_Delay(&htest->SwTimings.ticks_before_expected_peak, &hmcstim);
  
  /* Обратное включение диода */
  TESTER_Connect_Phase(&htest->SwPhaseReverse);
  
  /* Обратное включение на определенное время */
  TESTER_Delay(&htest->SwTimings.ticks_before_disconnect, &hmcstim);
  
  /* Отключение питания от диода */
  TESTER_Disconnect_AllPhases(&htest->SwPhaseReverse, &htest->SwPhaseForward);
  
  /* Обработка DMA */
  ADC_DMA_ReadForPeak(htest->adc, ADC_READ_TIMEOUT_MS);
  
  TESTER_LED_WaitForAction(&htest->leds->LED1);
  
  htest->DiodeReversePeakVolt = htest->adc->chAdc.U_Current;  
}


/**
  * @brief    Тест диодов: подключение прямого, а потом обратного напряжения
  */
void TESTER_TestDiode_SwitchConnection(TESTER_TestHandleTypeDef *htest)
{ 
  /* Отключение питания от диода */
  TESTER_Disconnect_AllPhases(&htest->SwPhaseReverse, &htest->SwPhaseForward);
  /* Очистка буферов каналов */
  ClearStruct(htest->adc->chAdc.ADC_Buff);
  
  /* Задержка, перед началом работы */
  TESTER_LED_TimeoutForStartTest(&htest->leds->LED1);
  TESTER_Delay(&htest->SwTimings.ticks_before_test, &hmcstim);
  TESTER_LED_TestingDiodeForward(&htest->leds->LED1);
  
  /* Прямое включение диода */
  TESTER_Connect_Phase(&htest->SwPhaseForward);
  
  /* Прямое включение на определенное время */
  msDelay(htest->SwTimings.msticks_for_forward);
  htest->DiodeForwardVolt = htest->adc->chAdc.U_Current;
  TESTER_LED_TestingDiodeReverse(&htest->leds->LED1);
    
  /* Переход из прямого включения в обратное */
  TESTER_Reconnect_TwoPhases(&htest->SwPhaseForward, &htest->SwPhaseReverse, &htest->SwTimings.ticks_deadtime);
  
  /* Обратное включение на определенное время */
  TESTER_Delay(&htest->SwTimings.ticks_before_disconnect, &hmcstim);
  
  /* Отключение питания от диода */
  TESTER_Disconnect_AllPhases(&htest->SwPhaseReverse, &htest->SwPhaseForward);
  
  /* Обработка DMA */
  ADC_DMA_ReadForPeak(htest->adc, ADC_READ_TIMEOUT_MS);
  
  TESTER_LED_WaitForAction(&htest->leds->LED1);
  
  htest->DiodeReversePeakVolt = htest->adc->chAdc.U_Current;  
}


/**
  * @brief    Тестирование одного ключа
  */
void TESTER_TestOneSwitch(GPIO_TypeDef *SwPort, uint32_t SwPin, uint8_t flag)
{
  if(flag)
  {
    TESTER_PhaseSwSingle_Set(SwPort, SwPin);
  }
  else
  {
    TESTER_PhaseSwSingle_Reset(SwPort, SwPin);
  }
}

/**
  * @brief    Подключить фазу с помощью ключей (пинов) в TESTER_PhaseSwitchTypeDef
  */
void TESTER_Connect_Phase(TESTER_PhaseSwitchTypeDef *PhaseSw)
{
  TESTER_PhaseSw_Set(PhaseSw);
}
/**
  * @brief    Отключить фазу с помощью ключей (пинов) в TESTER_PhaseSwitchTypeDef
  */
void TESTER_Disconnect_Phase(TESTER_PhaseSwitchTypeDef *PhaseSw)
{
  TESTER_PhaseSw_Reset(PhaseSw);
}

/**
  * @brief    Отключить фазу с помощью ключей (пинов) в TESTER_PhaseSwitchTypeDef
  */
void TESTER_Disconnect_AllPhases(TESTER_PhaseSwitchTypeDef *SwPhaseA, TESTER_PhaseSwitchTypeDef *SwPhaseB)
{
  TESTER_PhaseSw_Reset(SwPhaseA);
  TESTER_PhaseSw_Reset(SwPhaseB);
}
/**
  * @brief    Переключить две фазы с помощью ключей (пинов) в TESTER_PhaseSwitchTypeDef
  */
void TESTER_Reconnect_TwoPhases(TESTER_PhaseSwitchTypeDef *SwPhaseA, TESTER_PhaseSwitchTypeDef *SwPhaseB, TESTER_TicksDelayTypeDef *deadtime)
{
  /* Если дедтайм меньше 100 мкс, то сначала запускаем АЦП а потом дедтаймим */
  /* Потому что эта функция по длительности порядка 30 мкс, поэтому дедтайм получается не может быть меньше 30 мкс*/
  #ifndef RECONNECT_WITHOUT_DEADTIME
  if(deadtime->msdelay == 0)
  {
    if(deadtime->ticks < 100)
    {
  #endif //RECONNECT_WITHOUT_DEADTIME
      /* Включение АЦП */
      ADC_DMA_StartRead(hTestDiode.adc);
  
      /* Ожидается задержка, перед предполагаемым скачком */
      TESTER_Delay(&hTestDiode.SwTimings.ticks_before_expected_peak, &hmcstim);  
  #ifndef RECONNECT_WITHOUT_DEADTIME
    }
  }
  #endif //RECONNECT_WITHOUT_DEADTIME
    
  TESTER_PhaseSw_Reset(SwPhaseA);
  
  #ifndef RECONNECT_WITHOUT_DEADTIME
  /* Ожидается задержка дедтайм */
  TESTER_Delay(deadtime, &hmcstim);
  
  /* Включение АЦП */
  ADC_DMA_StartRead(hTestDiode.adc);    
  
  /* Ожидается задержка, перед предполагаемым скачком */
  TESTER_Delay(&hTestDiode.SwTimings.ticks_before_expected_peak, &hmcstim);  
  #endif //RECONNECT_WITHOUT_DEADTIME
  
  
  TESTER_PhaseSw_Set(SwPhaseB);
}


/**
* @brief    Формирование задержки (в микро или миллисекундная)
  */
void TESTER_Delay(TESTER_TicksDelayTypeDef *tickdelay, TIM_HandleTypeDef *htim)
{
  /* если миллисекундная задержка выключена */
  if(tickdelay->msdelay == 0)
  {
    htim->Instance->CNT = 1;
    uint32_t tickstart = HAL_GetTick();
    /* Задержка, в мкс */
    while(htim->Instance->CNT < tickdelay->ticks)
    {
      /* Если прошло уже больше секунды, а микросекундная задержка не закончилась - возврат */
      if(HAL_GetTick() - tickstart > 1000)
        return;
    }
  }
  /* если миллисекундная задержка включена */
  else
  {
    /* Задержка, в миллисекундах */
    msDelay(tickdelay->ticks);
  }
}