/**
**************************************************************************
* @file tester_func.h
* @brief Заголовочный файл для функций МЗКТ.
**************************************************************************
@details

*************************************************************************/

	
#ifndef _TESTER_FUNC_H_
#define _TESTER_FUNC_H_

#include "tester_adc_func.h"
#include "tester_interface_func.h"

/**
  * @brief Хендл микросекундного таймера
  */
#define hmcstim htim3 


#ifdef USE_HAL_GPIO_FUNCTIONS
#define TESTER_PhaseSw_Set(_sw_)      HAL_GPIO_WritePin((_sw_)->SW_Port, (_sw_)->SwHI_Pin | (_sw_)->SwLO_Pin, PHASE_CONNECT)
#define TESTER_PhaseSw_Reset(_sw_)    HAL_GPIO_WritePin((_sw_)->SW_Port, (_sw_)->SwHI_Pin | (_sw_)->SwLO_Pin, PHASE_DISCONNECT)

#define TESTER_PhaseSwSingle_Set(_swport_, _swpin_)      HAL_GPIO_WritePin((_swport_), (_swpin_), PHASE_CONNECT)
#define TESTER_PhaseSwSingle_Reset(_swport_, _swpin_)      HAL_GPIO_WritePin((_swport_), (_swpin_), PHASE_DISCONNECT)
#else //USE_HAL_GPIO_FUNCTIONS

#if (PHASE_CONNECT == 1) && (PHASE_DISCONNECT == 0)
#define TESTER_PhaseSw_Set(_sw_)      (_sw_)->SW_Port->BSRR = (_sw_)->SwHI_Pin | (_sw_)->SwLO_Pin
#define TESTER_PhaseSw_Reset(_sw_)    (_sw_)->SW_Port->BSRR = ((_sw_)->SwHI_Pin | (_sw_)->SwLO_Pin) << 16

#define TESTER_PhaseSwSingle_Set(_swport_, _swpin_)     (_swport_)->BSRR = (_swpin_)
#define TESTER_PhaseSwSingle_Reset(_swport_, _swpin_)   (_swport_)->BSRR = ((_swpin_) << 16)
#elif (PHASE_CONNECT == 0) && (PHASE_DISCONNECT == 1)
#define TESTER_PhaseSw_Set(_sw_)      (_sw_)->SW_Port->BSRR = ((_sw_)->SwHI_Pin | _sw_->SwLO_Pin) << 16
#define TESTER_PhaseSw_Reset(_sw_)    (_sw_)->SW_Port->BSRR = (_sw_)->SwHI_Pin | (_sw_)->SwLO_Pin

#define TESTER_PhaseSwSingle_Set(_swport_, _swpin_)     (_swport_)->BSRR = ((_swpin_) << 16)
#define TESTER_PhaseSwSingle_Reset(_swport_, _swpin_)   (_swport_)->BSRR = (_swpin_)
#endif //POWER_CONNECT

#endif //USE_HAL_GPIO_FUNCTIONS

extern TIM_HandleTypeDef htim3;

/**
  * @brief Структура для пинов, которые отвечают за ключи, которые подключают фазы
  */
typedef struct
{
  GPIO_TypeDef *SW_Port;      ///< Порт первого ключа для фазы
  uint32_t      SwHI_Pin;    ///< Пин первого ключа для фазы
  uint32_t      SwLO_Pin;    ///< Пин второго ключа для фазы
}TESTER_PhaseSwitchTypeDef;


/**
  * @brief Структура для тайминга с флагом миллисекундой или тиковой (пустой for) задержкой
  */
typedef struct
{
  unsigned  msdelay:1;
  uint32_t  ticks;
}TESTER_TicksDelayTypeDef;

/**
  * @brief Структура с таймингами для тестера 
  */
typedef struct
{
  uint32_t                  msticks_for_forward;        ///< миллисекундная задержка для положительного напряжения @ref TESTER_TestDiode_Forward
  TESTER_TicksDelayTypeDef  ticks_before_test;          ///< задержка перед началом тестирования (мс или мкс)
  TESTER_TicksDelayTypeDef  ticks_deadtime;             ///< задержка между включением АЦП и предполагаемым скачком напряжения, чтобы точно поймать его в буфере АЦП (мс или мкс)
  TESTER_TicksDelayTypeDef  ticks_before_expected_peak; ///< задержка между включением АЦП и предполагаемым скачком напряжения, чтобы точно поймать его в буфере АЦП (мс или мкс)
  TESTER_TicksDelayTypeDef  ticks_before_disconnect;    ///< задержка перед выключением фаз (мс или мкс)
}TESTER_SwitchTimingsTypeDef;

/**
  * @brief Структура-дескриптор для управления тестированием диодов
  */
typedef struct
{
  TESTER_ADCTypeDef *adc;                 ///< указатель на структуру АЦП тестера
  
  TESTER_SwitchTimingsTypeDef SwTimings;  ///< Тайминги для ключей
  
  TESTER_PhaseSwitchTypeDef SwPhaseForward;      ///< Пины ключей фаз для прямого включения
  TESTER_PhaseSwitchTypeDef SwPhaseReverse;      ///< Пины ключей фаз для обратного включения
  
  uint32_t          continuous_buff_size;
  
  float             DiodeForwardVolt;
  float             DiodeReversePeakVolt;
  
  TESTER_LEDsTypeDef *leds;
}TESTER_TestHandleTypeDef;
extern TESTER_TestHandleTypeDef hTestDiode;



/* Инициализация хендла тестера диодов */
void TESTER_HandleInit(TESTER_TestHandleTypeDef *htest, TESTER_LEDsTypeDef *leds);

/* Тест диодов: подключение прямого напряжения */
void TESTER_TestDiode_Forward(TESTER_TestHandleTypeDef *htest);
/* Тест диодов: подключение обратного напряжения */
void TESTER_TestDiode_Reverse(TESTER_TestHandleTypeDef *htest);
/* Тест диодов: подключение прямого, а потом обратного напряжения */
void TESTER_TestDiode_SwitchConnection(TESTER_TestHandleTypeDef *htest);

/* Тестирование одного ключа */
void TESTER_TestOneSwitch(GPIO_TypeDef *SwPort, uint32_t SwPin, uint8_t flag);
  
/* Подключить фазу с помощью ключей (пинов) в TESTER_PhaseSwitchTypeDef */
void TESTER_Connect_Phase(TESTER_PhaseSwitchTypeDef *PhaseSw);
/* Отключить фазу с помощью ключей (пинов) в TESTER_PhaseSwitchTypeDef */
void TESTER_Disconnect_Phase(TESTER_PhaseSwitchTypeDef *PhaseSw);
/* Отключить фазу с помощью ключей (пинов) в TESTER_PhaseSwitchTypeDef */
void TESTER_Disconnect_AllPhases(TESTER_PhaseSwitchTypeDef *SwPhaseA, TESTER_PhaseSwitchTypeDef *SwPhaseB);
/* Переключить две фазы с помощью ключей (пинов) в TESTER_PhaseSwitchTypeDef */
void TESTER_Reconnect_TwoPhases(TESTER_PhaseSwitchTypeDef *SwPhaseA, TESTER_PhaseSwitchTypeDef *SwPhaseB, TESTER_TicksDelayTypeDef *deadtime);
/* Формирование задержки (в микро или миллисекундная) */
void TESTER_Delay(TESTER_TicksDelayTypeDef *tickdelay, TIM_HandleTypeDef *htim);
#endif //_TESTER_FUNC_H_

