#include "tester_interface_func.h"


/**
  * @brief Инициализация структур интерфейса 
  */
void TESTER_InterfaceInit(TESTER_SwitchStartTypeDef *swstart, TESTER_LEDsTypeDef *leds)
{
}

/**
* @brief Включить индикацию таймаута старта при активации теста
	*/
void TESTER_LED_TimeoutForStartTest(TESTER_LEDTypeDef *led)
{
  TESTER_LED_Off(led);
}

/**
  * @brief Включить индикацию прямого подключения диода
	*/
void TESTER_LED_TestingDiodeForward(TESTER_LEDTypeDef *led)
{
  led->state = LED_IS_BLINKING;
  led->period = LED_FORWARD_DIODE_PERIOD;
}
/**
  * @brief Включить индикацию обратного подключения диода
	*/
void TESTER_LED_TestingDiodeReverse(TESTER_LEDTypeDef *led)
{
  led->state = LED_IS_BLINKING;
  led->period = LED_REVERSE_DIODE_PERIOD;
}
/**
  * @brief Выключить индикацию ожидания комманды
  * @details Сделано через моргание, чтобы понимать, что системные тики работают
	*/
void TESTER_LED_WaitForAction(TESTER_LEDTypeDef *led)
{
  led->state = LED_IS_BLINKING;
  led->period = LED_BLINK_AS_ON;
}


/**
  * @brief Считать состоянии кнопки запуска 
  */
uint8_t TESTER_ReadSwichStart(TESTER_SwitchStartTypeDef *swstart)
{

  if(HAL_GPIO_ReadPin(swstart->Sw_Port, swstart->Sw_Pin) == SW_ON)
  {
    if(swstart->tickprev == 0)
      swstart->tickprev = HAL_GetTick();
    
    if((HAL_GetTick() - swstart->tickprev) > swstart->Sw_FilterDelay)
    {
      if(HAL_GPIO_ReadPin(swstart->Sw_Port, swstart->Sw_Pin) == SW_ON)
      {
        return 1;
      }
      else
      {
        swstart->tickprev = 0;
        return 0;
      }
    }
  }
  return 0;
}
/**
  * @brief  Включить светодиод 
  */
void TESTER_LED_On(TESTER_LEDTypeDef *led)
{
  led->state = LED_IS_ON;
  HAL_GPIO_WritePin(led->LED_Port, led->LED_Pin, LED_ON);
}
/**
  * @brief  Выключить светодиод 
  */
void TESTER_LED_Off(TESTER_LEDTypeDef *led)
{
  led->state = LED_IS_OFF;
  HAL_GPIO_WritePin(led->LED_Port, led->LED_Pin, LED_OFF);
}

/**
  * @brief Активировать моргание светодиодом
	*/
void TESTER_LED_Blink_Start(TESTER_LEDTypeDef *led)
{
  led->state = LED_IS_BLINKING;
}
  
/**
  * @brief Моргание светодиодом
	*/
void TESTER_LEDBlink_Handle(TESTER_LEDTypeDef *led)
{
  if(led->state == LED_IS_BLINKING)
  {
    uint32_t tickcurrent = HAL_GetTick();
    if((tickcurrent - led->tickprev) > led->period)
    {
      HAL_GPIO_TogglePin(led->LED_Port, led->LED_Pin);
      led->tickprev = tickcurrent;
    }
  }
}
